/**************************************************************************//**
*   @file   ad5415.c
*   @brief  Implementation of AD5415 Driver for NIOS processor.
*   @author DBogdan (dragos.bogdan@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 158
******************************************************************************/
#define AD5415_GLOBALS

/*****************************************************************************/
/* Include Files															 */
/*****************************************************************************/
#include "ad5415.h"
#include <altera_avalon_pio_regs.h>	// PIO access macros
#include <altera_avalon_spi.h>		// SPI
#include <unistd.h>					// usleep();
#include <system.h>
#if (USE_AD5415 == 1)

/*****************************************************************************/
/* Private variables                                                         */
/*****************************************************************************/
unsigned char	txBuffer[4] = {0,};	// Tx SPI Buffer.
unsigned char   rxBuffer[4] = {0,};	// Rx SPI Buffer.
unsigned int 	controlWord	= 0;	// The word which will be sent to the AD5415 circuit as control register value.

/****************************************************************************/
/*  Aliases for the uC Probe data                                           */
/****************************************************************************/
volatile unsigned int* 	dataSlider			= &AD5415_data[0].uiData;	// Value of Data Slider.
volatile unsigned char* nopBothButton		= &AD5415_data[1].uchData;	// Position of the NopBoth  button.
volatile unsigned char* loadUpdateAButton	= &AD5415_data[2].uchData;	// Position of the LoadUpdateA  button.
volatile unsigned char* readbackAButton		= &AD5415_data[3].uchData;	// Position of the ReadbackA  button.
volatile unsigned char* loadAButton			= &AD5415_data[4].uchData;	// Position of the LoadA button.
volatile unsigned char* loadUpdateBButton	= &AD5415_data[5].uchData;	// Position of the LoadUpdateB button.
volatile unsigned char* readbackBButton		= &AD5415_data[6].uchData;	// Position of the ReadbackB button.
volatile unsigned char* loadBButton			= &AD5415_data[7].uchData;	// Position of the LoadB button.
volatile unsigned char* updateBothButton	= &AD5415_data[8].uchData;	// Position of the UpdateBoth button.
volatile unsigned char* loadBothButton		= &AD5415_data[9].uchData;	// Position of the LoadBoth button.
volatile unsigned int* 	readDacA			= &AD5415_data[10].uiData;	// Value of the DAC A read data.
volatile unsigned int* 	readDacB			= &AD5415_data[11].uiData;	// Value of the DAC B read data.
volatile unsigned char* sendControlButton	= &AD5415_data[12].uchData;	// Position of the SendControl button.
volatile unsigned char* sdo1Button			= &AD5415_data[13].uchData;	// Position of the SDO1 button.
volatile unsigned char* sdo2Button			= &AD5415_data[14].uchData;	// Position of the SDO2 button.
volatile unsigned char* dsyButton			= &AD5415_data[15].uchData;	// Position of the DSY button.
volatile unsigned char* hclrButton			= &AD5415_data[16].uchData;	// Position of the HCLR button.
volatile unsigned char* sckButton			= &AD5415_data[17].uchData;	// Position of the SCK button.
volatile unsigned char* clrButton			= &AD5415_data[18].uchData;	// Position of the CLR button.
volatile unsigned char* ldacButton			= &AD5415_data[19].uchData;	// Position of the LDAC button.
volatile unsigned char* clearZeroButton		= &AD5415_data[20].uchData;	// Position of the LDAC button.
volatile unsigned char* clearMidButton		= &AD5415_data[21].uchData;	// Position of the LDAC button.

/**************************************************************************//**
 * @brief Transmits 16 bits on SPI.
 *
 * @param value - Data which will be transmitted.
 *
 * @return None
******************************************************************************/
void AD5415_TxSpi(unsigned int value)
{
	txBuffer[0] = (unsigned char) ((value & 0x0000ff00) >> 8);
	txBuffer[1] = (unsigned char)  (value & 0x000000ff);
    alt_avalon_spi_command(AD5415_SPI,	// base address of SPI module
    					   0,           // slave number
    					   2,           // write_length
    					   txBuffer,    // *write_data
    					   0,           // read_length
    					   rxBuffer,    // *read_data
    					   0);          // flags
}

/**************************************************************************//**
 * @brief Receives 16 bits on SPI.
 *
 * @param None
 *
 * @return returnVal - Received data.
******************************************************************************/
int AD5415_RxSpi(void)
{
	unsigned int returnVal;
	txBuffer[0] = (unsigned char) 0;
	txBuffer[1] = (unsigned char) 0;
    alt_avalon_spi_command(AD5415_SPI,  // base address of SPI module
    					   0,           // slave number
    					   0,           // write_length
    					   txBuffer,    // *write_data
    					   2,           // read_length
    					   rxBuffer,    // *read_data
    					   0);			// flags
    returnVal = ((unsigned int) rxBuffer[0]) << 8;
    returnVal += (unsigned int) rxBuffer[1];

    return(returnVal);
}

/**************************************************************************//**
 * @brief Initializes the communication with the device.
 * 		  Initializes variables with values from the default interface.
 *
 * @param None.
 *
 * @return SUCCESS.
******************************************************************************/
int AD5415_Init(void)
{
	IOWR_ALTERA_AVALON_PIO_DATA(GPIO_BASE, 0);							// Reset data.
	IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, GPIO_LDAC | GPIO_CLR);	// Output.
	IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_LDAC);			// Clear LDAC pin (Active Low).
	IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_CLR);				// Set CLR pin (Active Low).
	SetCtrl(AD5415_SPI_CFG);											// Setup SPI interface.
	AD5415_TxSpi(0x9000);												// Disable daisy-chain.
	*clrButton = 1;

	return(SUCCESS);
}

/**************************************************************************//**
 * @brief Updates the device's monitored and controlled parameters
 *
 * @param : None.
 *
 * @return: SUCCESS.
******************************************************************************/
int AD5415_UpdateParameters(void)
{
	if(*clrButton)			// If the CLR button was pressed, toggle the state of the CLR pin.
	{
		IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_CLR);
	}
	else
	{
		IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_CLR);
	}
	if(*ldacButton)			// If the LDAC button was pressed, toggle the state of the LDAC pin.
	{
		IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_LDAC);
	}
	else
	{
		IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_LDAC);
	}
	if(*nopBothButton)		// If the NopBoth button was pressed, sends the NOP command to both DACs.
	{
		AD5415_TxSpi(0x0000 + *dataSlider);
		*nopBothButton = 0;
	}
	if(*loadUpdateAButton)	// If the LoadUpdateA button was pressed,
	{						// loads the slider value into the DAC A input register and update the DAC.
		AD5415_TxSpi(0x1000 + *dataSlider);
		*loadUpdateAButton = 0;
	}
	if(*readbackAButton)	// If the ReadbackA button was pressed, reads the DAC A value.
	{
		AD5415_TxSpi(0x2000);
		*readDacA = AD5415_RxSpi();
		*readbackAButton = 0;
	}
	if(*loadAButton)		// If the LoadA button was pressed, loads the slider value into the DAC A input register.
	{
		AD5415_TxSpi(0x3000 + *dataSlider);
		*loadAButton = 0;
	}
	if(*loadUpdateBButton)	// If the LoadUpdateB button was pressed,
	{						// loads the slider value into the DAC B input register and update the DAC.
		AD5415_TxSpi(0x4000 + *dataSlider);
		*loadUpdateBButton = 0;
	}
	if(*readbackBButton)	// If the ReadbackB button was pressed, reads the DAC B value.
	{
		AD5415_TxSpi(0x5000);
		*readDacB = AD5415_RxSpi();
		*readbackBButton = 0;
	}
	if(*loadBButton)		// If the LoadB button was pressed, loads the slider value into the DAC A input register.
	{
		AD5415_TxSpi(0x6000 + *dataSlider);
		*loadBButton = 0;
	}
	if(*updateBothButton)	// If the UpdateBoth button was pressed, updates both DACs.
	{
		AD5415_TxSpi(0x7000 + *dataSlider);
		*updateBothButton = 0;
	}
	if(*loadBothButton)		// If the LoadBoth button was pressed,
	{						// loads the slider value into the both DACs input registers.
		AD5415_TxSpi(0x8000 + *dataSlider);
		*loadBothButton = 0;
	}
	if(*clearZeroButton)// If the ClearZero button was pressed, sends the Clear DAC output to zero scale command.
	{
		AD5415_TxSpi(0xB000);
		*clearZeroButton = 0;
	}
	if(*clearMidButton)// If the ClearMid button was pressed, sends the Clear DAC output to midscale command.
	{
		AD5415_TxSpi(0xC000);
		*clearMidButton = 0;
	}
	if(*sendControlButton)	// If the SendControl button was pressed, updates the control register.
	{
		controlWord = (((*sckButton * 1) + (*hclrButton * 2) +
						(*dsyButton * 4) + (*sdo2Button * 8) +
						(*sdo1Button * 16)) << 7) + 0xD000;
		AD5415_TxSpi(controlWord);
		*sendControlButton = 0;
	}

	return(SUCCESS);
}
#endif /* USE_AD5415 */
